#!/usr/local/bin/perl
##
## Program: notifier.pl v1.1, Rule-based Notification Addon for Nagios(r)
## License: GPL
## Copyright (c) 2012 Yueh-Hung Liu (yuehung.liu@gmail.com)
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.
##


use Getopt::Long;
use File::Basename;
use XML::Simple;

my $data_type;
my $config_file = dirname($0)."/notifier.cfg";
my $config;
my $debug;
my $log_file;
my %commands;
my ( $contacts, $hosts, $services, $states, $timeperiods, $numbers, $methods, $extra );
my $found;
my $wildcard;
my $state;
my %methods_hash;
my $priority = 100;


if( ! GetOptions( "type|t=s" => \$data_type, "config|c=s" => \$config_file ) )
{
	print "usage: $0 -t {host|service} [-c <config file>]\n";
	exit 1;
}

if( $data_type ne "host" && $data_type ne "service" )
{
	print "usage: $0 -t {host|service} [-c <config file>]\n";
	exit 1;
}

open CONFIG, "< $config_file" or die "failed to open config file!";
close CONFIG;		#### just for testing config file ####

$config = XMLin($config_file);
$debug = $$config{'debug'};
$debug += 0;		#### convert to numeric ####
$log_file = $$config{'log_file'};
chomp $log_file;
open LOG, ">> $log_file" or die "failed to open log file!" if $debug;

print LOG "\ndate = $ENV{'NAGIOS_LONGDATETIME'}\n" if $debug;
print LOG "data type = $data_type\n" if $debug;
print LOG "config file = $config_file\n" if $debug;

foreach( split( /\n/, $$config{'commands'}{$data_type} ) )
{
	s/^[\s]*//;
	s/[\s]*$//;
	next if /^$/;

	chomp;
	/^(.+?)[\s]*=[\s]*(.*)$/;
	$commands{$1} = $2;
	$commands{$1} =~ s/\$(.+?)\$/$ENV{"NAGIOS_$1"}/g;
}

foreach( split( /\n/, $$config{'rules'}{$data_type} ) )
{
	s/^[\s]*//;
	s/[\s]*$//;
	next if /^$/;

	chomp;
	( $contacts, $hosts, $services, $states, $timeperiods, $numbers, $methods, $extra ) = split /[\s]*:[\s]*/;
	if( $data_type eq "host" && $services ne "-" || $data_type eq "service" && $services eq "-" )
	{
		next;
	}

	print LOG "#### matching rule ` $_ '....\n" if $debug;
	$wildcard = 0;

	$found = in_array( [ $ENV{"NAGIOS_CONTACTNAME"}, split( /,/, $ENV{"NAGIOS_CONTACTGROUPNAMES"} ) ], [ split( /[\s]*,[\s]*/, $contacts ) ] );
	if( ! $found )
	{
		print LOG "can't find contact ` $ENV{'NAGIOS_CONTACTNAME'}(groups=$ENV{'NAGIOS_CONTACTGROUPNAMES'}) ' in ` $contacts '....skip this rule\n" if $debug;
		next;
	}
	print LOG "found contact ` $ENV{'NAGIOS_CONTACTNAME'}(groups=$ENV{'NAGIOS_CONTACTGROUPNAMES'}) ' in ` $contacts ' [$found]\n" if $debug;
	$wildcard += $found-1;

	$found = in_array( [ $ENV{"NAGIOS_HOSTNAME"}, split( /,/, $ENV{"NAGIOS_HOSTGROUPNAMES"} ) ], [ split( /[\s]*,[\s]*/, $hosts ) ] );
	if( ! $found )
	{
		print LOG "can't find host ` $ENV{'NAGIOS_HOSTNAME'}(groups=$ENV{'NAGIOS_HOSTGROUPNAMES'}) ' in ` $hosts '....skip this rule\n" if $debug;
		next;
	}
	print LOG "found host ` $ENV{'NAGIOS_HOSTNAME'}(groups=$ENV{'NAGIOS_HOSTGROUPNAMES'}) ' in ` $hosts ' [$found]\n" if $debug;
	$wildcard += $found-1;

	if( $data_type eq "service" )
	{
		$found = in_array( [ $ENV{"NAGIOS_SERVICEDESC"}, split( /,/, $ENV{"NAGIOS_SERVICEGROUPNAMES"} ) ], [ split( /[\s]*,[\s]*/, $services ) ] );
		if( ! $found )
		{
			print LOG "can't find service ` $ENV{'NAGIOS_SERVICEDESC'}(groups=$ENV{'NAGIOS_SERVICEGROUPNAMES'}) ' in ` $services '....skip this rule\n" if $debug;
			next;
		}
		print LOG "found service ` $ENV{'NAGIOS_SERVICEDESC'}(groups=$ENV{'NAGIOS_SERVICEGROUPNAMES'}) ' in ` $services ' [$found]\n" if $debug;
		$wildcard += $found-1;
	}

	$state = $data_type eq "host" ? $ENV{"NAGIOS_HOSTSTATE"} : $ENV{"NAGIOS_SERVICESTATE"};
	$found = in_array( [ $state ], [ split( /[\s]*,[\s]*/, $states ) ] );
	if( ! $found )
	{
		print LOG "can't find state ` $state ' in ` $states '....skip this rule\n" if $debug;
		next;
	}
	print LOG "found state ` $state ' in ` $states ' [$found]\n" if $debug;
	$wildcard += $found-1;

	$found = in_timeperiod( $ENV{"NAGIOS_TIME"}, split( /[\s]*,[\s]*/, $timeperiods ) );
	if( ! $found )
	{
		print LOG "can't find time ` $ENV{'NAGIOS_TIME'} ' in ` $timeperiods '....skip this rule\n" if $debug;
		next;
	}
	print LOG "found time ` $ENV{'NAGIOS_TIME'} ' in ` $timeperiods ' [$found]\n" if $debug;
	$wildcard += $found-1;

	$found = in_number( $ENV{"NAGIOS_NOTIFICATIONNUMBER"}, split( /[\s]*,[\s]*/, $numbers ) );
	if( ! $found )
	{
		print LOG "can't find notification number ` $ENV{'NAGIOS_NOTIFICATIONNUMBER'} ' in ` $numbers '....skip this rule\n" if $debug;
		next;
	}
	print LOG "found notification number ` $ENV{'NAGIOS_NOTIFICATIONNUMBER'} ' in ` $numbers ' [$found]\n" if $debug;
	$wildcard += $found-1;

	if( ! $extra || $extra =~ /^\*$/)
	{
		$found = 2;
	}
	else
	{
		print LOG "extra matching with ` $extra '....\n" if $debug;
		$extra =~ /^\$(.+?)\$[\s]*=[\s]*(.*)$/;
		$found = in_array( [ $ENV{"NAGIOS_$1"} ], [ split( /[\s]*,[\s]*/, $2 ) ] );
		if( ! $found )
		{
			print LOG "can't find value ` $ENV{\"NAGIOS_$1\"} ' in ` $2 '....skip this rule\n" if $debug;
			next;
		}
		print LOG "found value ` $ENV{\"NAGIOS_$1\"} ' in ` $2 ' [$found]\n" if $debug;
	}
	$wildcard += $found-1;

	if( $wildcard > $priority )
	{
		print LOG "current priority = $priority; new priority = $wildcard....ignore this rule\n" if $debug;
	}
	elsif( $wildcard == $priority )
	{
		print LOG "current priority = $priority; new priority = $wildcard....merge this rule\n" if $debug;
		foreach( split( /[\s]*,[\s]*/, $methods ) )
		{
			$methods_hash{$_} = 1;
		}
	}
	else
	{
		print LOG "current priority = $priority; new priority = $wildcard....replace with this rule\n" if $debug;
		$priority = $wildcard;
		%methods_hash = ();
		foreach( split( /[\s]*,[\s]*/, $methods ) )
		{
			$methods_hash{$_} = 1;
		}
	}
}

if( ! exists $methods_hash{'-'} )
{
	if( exists $methods_hash{'*'} )
	{
		%methods_hash = ();
		foreach( keys %commands )
		{
			$methods_hash{$_} = 1;
		}
	}

	foreach( keys %methods_hash )
	{
		print LOG "#### notify $data_type by $_\n" if $debug;
		notify( $_ );
	}
}
else
{
	print LOG "#### silence\n" if $debug;
}

close LOG if $debug;



#######################
#
##### subroutines #####
#
#######################


#### 0: not fount; 1: exactly match; 1.5: group match; 2: always match ####
sub in_array
{
	my ( $items, $array ) = @_;		#### $items[0] is the item, $items[1] ~ $items[n] are belonged groups

	print LOG "finding ` @$items ' in ` @$array '....\n" if $debug;
	foreach my $match ( @$array )
	{
		return 2 if $match =~ /^\*$/;

		for( my $i = 0; $i <= $#$items; $i++)
		{
			if( $match =~ s/\*/.*/g )
			{
				return 1.5 if $$items[$i] =~ /^$match$/;
			}
			else
			{
				return ( $i > 0 ? 1.5 : 1 ) if $$items[$i] =~ /^$match$/;
			}
		}
	}

	return 0;
}

sub in_timeperiod
{
	my ( $time, @array ) = @_;

	my $start;
	my $end;

	print LOG "finding ` $time ' in ` @array '....\n" if $debug;
	$time =~ s/[\s]*:[\s]*//g;
	foreach( @array )
	{
		return 2 if /^\*$/;

		( $start, $end ) = split /[\s]*-[\s]*/;
		$start .= "00";
		$end .= "00";
		if( $start > $end )
		{
			return 1 if $time >= $start || $time <= $end;
		}
		else
		{
			return 1 if $time >= $start && $time <= $end;
		}
	}

	return 0;
}

sub in_number
{
	my ( $number, @array ) = @_;

	my $start;
	my $end;

	print LOG "finding ` $number ' in ` @array '....\n" if $debug;
	foreach( @array )
	{
		return 2 if /^\*$/;

		if ( /-/ )
		{
			( $start, $end ) = split /[\s]*-[\s]*/;
		}
		else
		{
			$start = $end = $_;
		}

		$end = $number if $end == 0;
		return 1 if $number >= $start && $number <= $end;
	}

	return 0;
}

sub notify
{
	my ( $method ) = @_;

	print LOG "command = $commands{$method}\n" if $debug;
	qx/$commands{$method}/;
}

